/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef LOG4CXX_THREAD_USERNAME_PATTERN_CONVERTER_H
#define LOG4CXX_THREAD_USERNAME_PATTERN_CONVERTER_H

#include <log4cxx/pattern/loggingeventpatternconverter.h>
#include <log4cxx/spi/loggingevent.h>

namespace LOG4CXX_NS
{

namespace pattern
{

class LOG4CXX_EXPORT ThreadUsernamePatternConverter : public LoggingEventPatternConverter
{

    public:
	    DECLARE_LOG4CXX_PATTERN(ThreadUsernamePatternConverter)
	    BEGIN_LOG4CXX_CAST_MAP()
	    LOG4CXX_CAST_ENTRY(ThreadUsernamePatternConverter)
	    LOG4CXX_CAST_ENTRY_CHAIN(LoggingEventPatternConverter)
	    END_LOG4CXX_CAST_MAP()

		ThreadUsernamePatternConverter();

	    static PatternConverterPtr newInstance(
	            const std::vector<LogString>& options);

	    void format(const spi::LoggingEventPtr& event,
	            LogString& toAppendTo,
	            LOG4CXX_NS::helpers::Pool& p) const override;
};

}
}

#endif

